@@ -1,7 +1,10 @@
 import traceback
 
+import sys
+
 from whatap.conf.configure import Configure as conf
-from whatap.trace import UdpSession, PacketTypeEnum
+from whatap.net.packet_type_enum import PacketTypeEnum
+from whatap.net.udp_session import UdpSession
 from whatap.trace.trace_context import TraceContext
 from whatap.trace.trace_context_manager import TraceContextManager
 from whatap.util.date_util import DateUtil
@@ -46,16 +49,16 @@ def start_interceptor(ctx):
     start_time = DateUtil.nowSystem()
     ctx.start_time = start_time
     
-    
     datas = [ctx.host,
              ctx.service_name,
              ctx.remoteIp,
              ctx.userAgentString,
              ctx.referer,
              ctx.userid,
-             ctx.isStaticContents]
+             ctx.isStaticContents
+             ]
     
-    UdpSession.send_packet(PacketTypeEnum.PACKET_REQUEST, ctx, datas)
+    UdpSession.send_packet(PacketTypeEnum.TX_START, ctx, datas)
     
     return ctx
 
@@ -71,10 +74,10 @@ def end_interceptor(thread_id=None):
     start_time = DateUtil.nowSystem()
     ctx.start_time = start_time
     
-    datas = [ctx.host, ctx.service_name, ctx.mtid, ctx.mdepth, ctx.mcaller]
+    datas = [ctx.host, ctx.service_name, ctx.mtid, ctx.mdepth, ctx.mcaller_txid,
+             ctx.mcaller_pcode, ctx.mcaller_spec, ctx.mcaller_url]
     ctx.elapsed = DateUtil.nowSystem() - start_time
-    UdpSession.send_packet(PacketTypeEnum.PACKET_REQUEST_END, ctx,
-                           datas)
+    UdpSession.send_packet(PacketTypeEnum.TX_END, ctx, datas)
 
 
 def interceptor(rn_environ, *args, **kwargs):
@@ -83,18 +86,10 @@ def interceptor(rn_environ, *args, **kwargs):
     fn, environ = rn_environ
     
     ctx = TraceContext()
-
+    
     ctx.host = environ.get('HTTP_HOST', '').split(':')[0]
     ctx.service_name = environ.get('PATH_INFO', '')
     
-    query_string = environ.get('QUERY_STRING', '')
-    if query_string:
-        ctx.service_name += '?{}'.format(query_string)
-    
-    if ctx.service_name.find('.') > -1 and ctx.service_name.split('.')[
-        1] in conf.web_static_content_extensions:
-        ctx.isStaticContents = 'true'
-    
     ctx.remoteIp = environ.get('REMOTE_ADDR', '')
     ctx.userAgentString = environ.get('HTTP_USER_AGENT', '')
     ctx.referer = environ.get('HTTP_REFERER''', '')
@@ -102,15 +97,23 @@ def interceptor(rn_environ, *args, **kwargs):
     if conf.trace_user_enabled:
         if conf.trace_user_using_ip:
             ctx.userid = ctx.remoteIp
-            pass
         else:
             ctx.userid = ctx.remoteIp  # TODO cookie
-            
-    mstt = environ.get('HTTP_{}'.format(conf._trace_mtrace_caller_key.upper().replace('-', '_')), '')
+    
+    mstt = environ.get('HTTP_{}'.format(
+        conf._trace_mtrace_caller_key.upper().replace('-', '_')), '')
     
     if mstt:
         ctx.setTransfer(mstt)
-        myid = environ.get('HTTP_{}'.format(conf._trace_mtrace_callee_key.upper().replace('-', '_')), '')
+        if conf.stat_mtrace_enabled:
+            val = environ.get('HTTP_{}'.format(
+                conf._trace_mtrace_info_key.upper().replace('-', '_')), '')
+            if val and len(val):
+                ctx.setTransferInfo(val)
+            pass
+
+        myid = environ.get('HTTP_{}'.format(
+            conf._trace_mtrace_callee_key.upper().replace('-', '_')), '')
         if myid:
             ctx.setTxid(myid)
     
@@ -120,6 +123,14 @@ def interceptor(rn_environ, *args, **kwargs):
     
     ctx = TraceContextManager.getLocalContext()
     if ctx:
+        query_string = environ.get('QUERY_STRING', '')
+        if query_string:
+            ctx.service_name += '?{}'.format(query_string)
+
+        if ctx.service_name.find('.') > -1 and ctx.service_name.split('.')[
+            1] in conf.web_static_content_extensions:
+            ctx.isStaticContents = 'true'
+
         if getattr(callback, 'status_code', None):
             status_code = callback.status_code
             errors = [callback.reason_phrase, callback.__class__.__name__]
@@ -134,27 +145,104 @@ def interceptor(rn_environ, *args, **kwargs):
             
             text = ''
             for key in keys:
-                text += '{}={}\n'.format(key.split('HTTP_')[1].lower(), environ[key])
-    
-            datas = [' ', 'HTTP-HEADERS', text]
+                text += '{}={}\n'.format(key.split('HTTP_')[1].lower(),
+                                         environ[key])
+            
+            datas = ['HTTP-HEADERS', 'HTTP-HEADERS', text]
             ctx.start_time = DateUtil.nowSystem()
-            UdpSession.send_packet(PacketTypeEnum.PACKET_MSG, ctx, datas)
-    
+            UdpSession.send_packet(PacketTypeEnum.TX_MSG, ctx, datas)
+        
         end_interceptor()
     return callback
 
 
 def interceptor_error(status_code, errors):
-    # errors: 'error_type', 'error_message'
     ctx = TraceContextManager.getLocalContext()
     ctx.status = int(status_code / 100)
     if ctx.status >= 4 and not ctx.error:
         ctx.error = 1
-        try:
-            errors.append(traceback.format_exc())
-        except:
-            errors.append('')
-        UdpSession.send_packet(PacketTypeEnum.PACKET_ERROR, ctx, errors)
+        
+        error = ''
+        frame = sys._current_frames().get(ctx.thread.ident)
+        if not frame:
+            return
+        
+        for stack in traceback.extract_stack(frame):
+            line = stack[0]
+            line_num = stack[1]
+            
+            if line.find('/whatap/trace') > -1 or line.find(
+                    '/threading.py') > -1:
+                continue
+            error += '{} (line {})\n'.format(line, line_num)
+        
+        errors.append(error)
+        
+        # errors.append(''.join(traceback.format_list(traceback.extract_stack(sys._current_frames()[ctx.thread.ident]))))
+        UdpSession.send_packet(PacketTypeEnum.TX_ERROR, ctx, errors)
+
+
+def interceptor_step_error(e):
+    ctx = TraceContextManager.getLocalContext()
+    ctx.error_step = e
+    if not ctx.error:
+        ctx.error = 1
+    
+    errors = []
+    errors.append(e.__class__.__name__)
+    errors.append(str(e.args[0]))
+    
+    error = ''
+    frame = sys._current_frames().get(ctx.thread.ident)
+    if not frame:
+        return
+    
+    for stack in traceback.extract_stack(frame):
+        line = stack[0]
+        line_num = stack[1]
+        
+        if line.find('/whatap/trace') > -1 or line.find('/threading.py') > -1:
+            continue
+        error += '{} (line {})\n'.format(line, line_num)
+    
+    errors.append(error)
+    # errors.append(''.join(traceback.format_list(traceback.extract_stack(sys._current_frames()[ctx.thread.ident]))))
+    UdpSession.send_packet(PacketTypeEnum.TX_ERROR, ctx, errors)
+
+
+def interceptor_httpc_request(fn, httpc_url, *args, **kwargs):
+    ctx = TraceContextManager.getLocalContext()
+    if not ctx or ctx.active_httpc_hash:
+        return fn(*args, **kwargs)
+
+    param = None
+    if httpc_url.find('?') > -1:
+        httpc_url, param = httpc_url.split('?')
+
+    start_time = DateUtil.nowSystem()
+    ctx.start_time = start_time
+    ctx.httpc_url = httpc_url
+    ctx.active_httpc_hash = ctx.httpc_url
+    
+    try:
+        callback = fn(*args, **kwargs)
+        return callback
+    except Exception as e:
+        interceptor_step_error(e)
+    finally:
+        datas = [ctx.httpc_url, ctx.mcallee]
+        ctx.elapsed = DateUtil.nowSystem() - start_time
+        UdpSession.send_packet(PacketTypeEnum.TX_HTTPC, ctx, datas)
+
+        if conf.profile_http_parameter_enabled:
+            param = args[1].body or param
+            if param:
+                datas = ['HTTP-PARAMETERS', args[1].method, param]
+                ctx.start_time = DateUtil.nowSystem()
+                UdpSession.send_packet(PacketTypeEnum.TX_SECURE_MSG, ctx, datas)
+
+        ctx.active_httpc_hash = 0
+        ctx.httpc_url = None
 
 
 def interceptor_db_con(fn, db_type, *args, **kwargs):
@@ -186,7 +274,7 @@ def interceptor_db_con(fn, db_type, *args, **kwargs):
     
     datas = [text]
     ctx.elapsed = DateUtil.nowSystem() - start_time
-    UdpSession.send_packet(PacketTypeEnum.PACKET_DB_CONN, ctx, datas)
+    UdpSession.send_packet(PacketTypeEnum.TX_DB_CONN, ctx, datas)
     
     return callback
 
@@ -211,24 +299,11 @@ def interceptor_db_execute(fn, *args, **kwargs):
         callback = fn(*args, **kwargs)
         return callback
     except Exception as e:
-        ctx.error_step = e
-        if not ctx.error:
-            ctx.error = 1
-            
-        errors = []
-        errors.append(e.__class__.__name__)
-        errors.append(e.args[1] if len(e.args) > 1 else \
-                          e.args[0])
-        try:
-            errors.append(traceback.format_exc())
-        except:
-            errors.append('')
-        UdpSession.send_packet(PacketTypeEnum.PACKET_ERROR, ctx, errors)
-        
+        interceptor_step_error(e)
     finally:
         datas = [ctx.lctx.get('dbc', ''), query]
         ctx.elapsed = DateUtil.nowSystem() - start_time
-        UdpSession.send_packet(PacketTypeEnum.PACKET_DB_SQL, ctx,
+        UdpSession.send_packet(PacketTypeEnum.TX_SQL, ctx,
                                datas)
         
         count = self.rowcount
@@ -236,7 +311,7 @@ def interceptor_db_execute(fn, *args, **kwargs):
             desc = '{0}: {1}'.format('Fetch count', count)
             datas = [' ', ' ', desc]
             ctx.elapsed = 0
-            UdpSession.send_packet(PacketTypeEnum.PACKET_MSG, ctx, datas)
+            UdpSession.send_packet(PacketTypeEnum.TX_MSG, ctx, datas)
         
         ctx.active_sqlhash = 0
 
@@ -256,11 +331,13 @@ def interceptor_db_close(fn, *args, **kwargs):
     text = 'DB: Close Connection.'
     datas = [' ', ' ', text]
     ctx.elapsed = DateUtil.nowSystem() - start_time
-    UdpSession.send_packet(PacketTypeEnum.PACKET_MSG, ctx, datas)
+    UdpSession.send_packet(PacketTypeEnum.TX_MSG, ctx, datas)
     return callback
 
 
 check_seq = 1
+
+
 def inter_tx_trace_auto_on(ctx):
     try:
         if isinstance(conf.mtrace_rate, str):
@@ -310,14 +387,19 @@ def inter_tx_trace_auto_on(ctx):
             elif rate == 1:
                 if check_seq % 10 == 0:
                     ctx.mtid = TraceContextManager.getId()
-                    
-def transfer(ctx, headers):
+
+
+def transfer(headers):
+    ctx = TraceContextManager.getLocalContext()
+    
     if not ctx.mtid:
         inter_tx_trace_auto_on(ctx)
-        
+    
     if ctx.mtid:
-        headers[conf._trace_mtrace_caller_key]=ctx.transfer()
-        ctx.mcallee=TraceContextManager.getId()
-        headers[conf._trace_mtrace_callee_key]=Hexa32.toString32(ctx.mcallee)
-    return headers
+        headers[conf._trace_mtrace_caller_key] = ctx.transfer()
+        if conf.stat_mtrace_enabled:
+            headers[conf._trace_mtrace_info_key] = ctx.transferInfo()
 
+        ctx.mcallee = TraceContextManager.getId()
+        headers[conf._trace_mtrace_callee_key] = Hexa32.toString32(ctx.mcallee)
+    return headers
